@@ -30,31 +30,42 @@ def get_pkl_from_hdfs(pkl_path):
 def get_spark(scale=0):
     import os
     from pyspark.sql import SparkSession
+    import uuid
+    tmp_uuid = str(uuid.uuid4())
+    app_name = f"skt-{os.environ.get('USER', 'default')}-{tmp_uuid}"
     os.environ['ARROW_PRE_0_15_IPC_FORMAT'] = '1'
     if scale in [1, 2, 3, 4]:
         spark = SparkSession \
             .builder \
+            .config('spark.app.name', app_name) \
             .config('spark.driver.memory', f'{scale*8}g') \
-            .config('spark.executor.memory', f'{scale*2}g') \
-            .config('spark.executor.number', f'{scale*8}g') \
+            .config('spark.executor.memory', f'{scale*3}g') \
+            .config('spark.executor.instances', f'{scale*8}') \
             .config('spark.driver.maxResultSize', f'{scale*4}g') \
             .config('spark.rpc.message.maxSize', '1024') \
             .config('spark.yarn.queue', 'airflow_job') \
             .config('spark.ui.enabled', 'false') \
             .config('spark.port.maxRetries', '128') \
+            .config('spark.executorEnv.ARROW_PRE_0_15_IPC_FORMAT', '1') \
+            .config('spark.yarn.appMasterEnv.ARROW_PRE_0_15_IPC_FORMAT', '1') \
             .enableHiveSupport() \
             .getOrCreate()
     else:
         spark = SparkSession \
             .builder \
-            .config('spark.driver.memory', '4g') \
-            .config('spark.executor.memory', '4g') \
+            .config('spark.app.name', app_name) \
+            .config('spark.driver.memory', '6g') \
+            .config('spark.executor.memory', '8g') \
             .config('spark.shuffle.service.enabled', 'true') \
             .config('spark.dynamicAllocation.enabled', 'true') \
             .config('spark.dynamicAllocation.maxExecutors', '200') \
+            .config('spark.driver.maxResultSize', f'6g') \
+            .config('spark.rpc.message.maxSize', '1024') \
             .config('spark.yarn.queue', 'airflow_job') \
             .config('spark.ui.enabled', 'false') \
             .config('spark.port.maxRetries', '128') \
+            .config('spark.executorEnv.ARROW_PRE_0_15_IPC_FORMAT', '1') \
+            .config('spark.yarn.appMasterEnv.ARROW_PRE_0_15_IPC_FORMAT', '1') \
             .enableHiveSupport() \
             .getOrCreate()
     spark.conf.set('spark.sql.execution.arrow.enabled', 'true')
@@ -140,3 +151,15 @@ def slack_send(
     r.raise_for_status()
     if not r.json()['ok']:
         raise Exception(r.json())
+
+
+def get_github_util():
+    from skt.github_utils import GithubUtil
+    github_token = get_secrets('github/sktaiflow')['token']
+    proxy = get_secrets('proxy')['proxy']
+    proxies = {
+        'http': proxy,
+        'https': proxy,
+    }
+    g = GithubUtil(github_token, proxies)
+    return g
